@@ -48,6 +48,9 @@ QString DialogSettings::FolderParameterDefaultValue;
 QString DialogSettings::FileParameterDefaultPath;
 int DialogSettings::_previewTimeout = 16;
 
+QIcon DialogSettings::AddIcon;
+QIcon DialogSettings::RemoveIcon;
+
 // TODO : Make DialogSetting a view of a Settings class
 
 DialogSettings::DialogSettings(QWidget * parent) : QDialog(parent), ui(new Ui::DialogSettings)
@@ -95,7 +98,7 @@ DialogSettings::DialogSettings(QWidget * parent) : QDialog(parent), ui(new Ui::D
 
   ui->rbLeftPreview->setChecked(_previewPosition == MainWindow::PreviewOnLeft);
   ui->rbRightPreview->setChecked(_previewPosition == MainWindow::PreviewOnRight);
-  const bool savedDarkTheme = QSettings().value("Config/DarkTheme", false).toBool();
+  const bool savedDarkTheme = QSettings().value(DARK_THEME_KEY, false).toBool();
   ui->rbDarkTheme->setChecked(savedDarkTheme);
   ui->rbDefaultTheme->setChecked(!savedDarkTheme);
   ui->cbNativeColorDialogs->setChecked(_nativeColorDialogs);
@@ -153,7 +156,7 @@ void DialogSettings::loadSettings()
   } else {
     _previewPosition = MainWindow::PreviewOnRight;
   }
-  _darkThemeEnabled = settings.value("Config/DarkTheme", false).toBool();
+  _darkThemeEnabled = settings.value(DARK_THEME_KEY, false).toBool();
   _languageCode = settings.value("Config/LanguageCode", QString()).toString();
   _nativeColorDialogs = settings.value("Config/NativeColorDialogs", false).toBool();
   _updatePeriodicity = settings.value(INTERNET_UPDATE_PERIODICITY_KEY, INTERNET_DEFAULT_PERIODICITY).toInt();
@@ -162,6 +165,8 @@ void DialogSettings::loadSettings()
   _logosAreVisible = settings.value("LogosAreVisible", true).toBool();
   _previewTimeout = settings.value("PreviewTimeout", 16).toInt();
   _outputMessageMode = static_cast<GmicQt::OutputMessageMode>(settings.value("OutputMessageMode", GmicQt::DefaultOutputMessageMode).toInt());
+  AddIcon = LOAD_ICON("list-add");
+  RemoveIcon = LOAD_ICON("list-remove");
 }
 
 int DialogSettings::previewTimeout()
@@ -211,7 +216,7 @@ void DialogSettings::done(int r)
 {
   QSettings settings;
   saveSettings(settings);
-  settings.setValue("Config/DarkTheme", ui->rbDarkTheme->isChecked());
+  settings.setValue(DARK_THEME_KEY, ui->rbDarkTheme->isChecked());
   settings.setValue("Config/LanguageCode", ui->languageSelector->selectedLanguageCode());
   QDialog::done(r);
 }
