@@ -43,7 +43,7 @@ public:
   void setValue(const QString & value) override;
   void reset() override;
   bool initFromText(const char * text, int & textLength) override;
-  bool isQuoted() const;
+  bool isQuoted() const override;
 public slots:
   void onButtonPressed();
 
