@@ -37,6 +37,7 @@
 #include <QWidget>
 #include <cmath>
 #include <cstdio>
+#include <cstdlib>
 #include "Common.h"
 #include "DialogSettings.h"
 #include "HtmlTranslator.h"
@@ -59,6 +60,8 @@ PointParameter::PointParameter(QObject * parent) : AbstractParameter(parent, tru
   _connected = false;
   _defaultRemovedStatus = false;
   _radius = KeypointList::Keypoint::DefaultRadius;
+  _visible = true;
+  _keepOpacityWhenSelected = false;
   setRemoved(false);
 }
 
@@ -68,8 +71,16 @@ PointParameter::~PointParameter()
   delete _rowCell;
 }
 
+bool PointParameter::isVisible() const
+{
+  return _visible;
+}
+
 void PointParameter::addTo(QWidget * widget, int row)
 {
+  if (!_visible) {
+    return;
+  }
   QGridLayout * grid = dynamic_cast<QGridLayout *>(widget->layout());
   if (!grid) {
     return;
@@ -123,9 +134,9 @@ void PointParameter::addTo(QWidget * widget, int row)
 void PointParameter::addToKeypointList(KeypointList & list) const
 {
   if (_removable && _removed) {
-    list.add(KeypointList::Keypoint(_color, _removable, _burst, _radius));
+    list.add(KeypointList::Keypoint(_color, _removable, _burst, _radius, _keepOpacityWhenSelected));
   } else {
-    list.add(KeypointList::Keypoint(_position.x(), _position.y(), _color, _removable, _burst, _radius));
+    list.add(KeypointList::Keypoint(_position.x(), _position.y(), _color, _removable, _burst, _radius, _keepOpacityWhenSelected));
   }
 }
 
@@ -190,16 +201,17 @@ void PointParameter::setValue(const QString & value)
 void PointParameter::reset()
 {
   enableNotifications(false);
-  _spinBoxX->setValue(_defaultPosition.rx());
-  _spinBoxY->setValue(_defaultPosition.ry());
-
+  if (_spinBoxX) {
+    _spinBoxX->setValue(_defaultPosition.rx());
+    _spinBoxY->setValue(_defaultPosition.ry());
+  }
   if (_removeButton && _removable) {
     _removeButton->setChecked((_removed = _defaultRemovedStatus));
   }
   enableNotifications(true);
 }
 
-// P = point(x,y,removable{0,1},burst{0,1},r,g,b,a,radius)
+// P = point(x,y,removable{(0),1},burst{(0),1},r,g,b,a{negative->keepOpacityWhenSelected},radius,widget_visible{0|(1)})
 bool PointParameter::initFromText(const char * text, int & textLength)
 {
   QList<QString> list = parseText("point", text, textLength);
@@ -215,6 +227,8 @@ bool PointParameter::initFromText(const char * text, int & textLength)
   _burst = false;
   _removable = false;
   _radius = KeypointList::Keypoint::DefaultRadius;
+  _visible = true;
+  _keepOpacityWhenSelected = false;
 
   float x = 50.0;
   float y = 50.0;
@@ -311,7 +325,10 @@ bool PointParameter::initFromText(const char * text, int & textLength)
     if (!ok) {
       return false;
     }
-    _color.setAlpha(alpha);
+    if (params[7].trimmed().startsWith("-") || (alpha < 0)) {
+      _keepOpacityWhenSelected = true;
+    }
+    _color.setAlpha(std::abs(alpha));
   }
 
   if (params.size() >= 9) {
@@ -329,6 +346,10 @@ bool PointParameter::initFromText(const char * text, int & textLength)
     _radius = radius;
   }
 
+  if ((params.size() >= 10) && (params[9].trimmed() == "0")) {
+    _visible = false;
+  }
+
   _position = _defaultPosition;
   return true;
 }
@@ -381,7 +402,7 @@ int PointParameter::randomChannel()
 
 void PointParameter::connectSpinboxes()
 {
-  if (_connected) {
+  if (_connected || !_spinBoxX) {
     return;
   }
   connect(_spinBoxX, SIGNAL(valueChanged(double)), this, SLOT(onSpinBoxChanged()));
@@ -394,7 +415,7 @@ void PointParameter::connectSpinboxes()
 
 void PointParameter::disconnectSpinboxes()
 {
-  if (!_connected) {
+  if (!_connected || !_spinBoxX) {
     return;
   }
   _spinBoxX->disconnect(this);
