@@ -27,6 +27,7 @@
 #include <QGridLayout>
 #include <QSizePolicy>
 #include "Common.h"
+#include "DialogSettings.h"
 
 SeparatorParameter::SeparatorParameter(QObject * parent) : AbstractParameter(parent, false), _frame(0) {}
 
@@ -49,6 +50,9 @@ void SeparatorParameter::addTo(QWidget * widget, int row)
   _frame->setSizePolicy(sizePolicy);
   _frame->setFrameShape(QFrame::HLine);
   _frame->setFrameShadow(QFrame::Sunken);
+  if (DialogSettings::darkThemeEnabled()) {
+    _frame->setStyleSheet("QFrame{ border-top: 0px none #a0a0a0; border-bottom: 2px solid rgb(160,160,160);}");
+  }
   grid->addWidget(_frame, row, 0, 1, 3);
 }
 
