@@ -72,7 +72,9 @@ public:
 public slots:
   void onUpdateDownloadsFinished(int status);
   void onApplyClicked();
+  void onPreviewUpdateRequested(bool synchronous);
   void onPreviewUpdateRequested();
+  void onPreviewKeypointsEvent(unsigned int flags, unsigned long time);
   void onFullImageProcessingDone();
   void expandOrCollapseFolders();
   void search(QString);
@@ -99,6 +101,7 @@ public slots:
   void onEscapeKeyPressed();
   void onPreviewImageAvailable();
   void onPreviewError(QString message);
+  void onParametersChanged();
 
 protected:
   void timerEvent(QTimerEvent *);
@@ -158,6 +161,8 @@ private:
   QVector<QWidget *> _filterUpdateWidgets;
   FiltersPresenter * _filtersPresenter;
   GmicProcessor _processor;
+  ulong _lastPreviewKeypointBurstUpdateTime;
+  bool _keypointBurstEnabled;
 };
 
 #endif // _GMIC_QT_MAINWINDOW_H_
